Summary: This package configures a host to use local dns caching 
Name: zz_local_dns_cache
Version: 2.0
Release: 2
License: GPL
Group: Fermi
URL: http://www.fnal.gov/cd/UNIX/linux/
Vendor: Fermilab
Packager: Jason Harrington
requires: bind
requires: bind-chroot
BuildRoot: %{_tmppath}/%{name}-%{version}
BuildArch: noarch


%description
Installing this package will reconfigure named to run as a caching nameserver which will only allow connections from localhost. /etc/resolv.conf will be modified to call on localhost as primary nameserver. /etc/dhcp/dhclient.conf will be modified to try and preserve nameserver priority across reboots.


%files


%post
# make sure named can write to master zones
if [ -x /usr/sbin/setsebool ]
then
    /usr/sbin/setsebool -P named_write_master_zones 1
fi

# get list of non-local nameservers from /etc/resolv.conf
ndns=`cat /etc/resolv.conf | grep -E '^[[:space:]]*nameserver' | grep -v 127.0.0.1 | sed -r 's/^[[:space:]]*nameserver[[:space:]]*([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}).*/\1/g' | tr '\n' ';'`
# make sure we have something that makes sense (non-zero length)
if [ -z "$ndns" ]
then
    ndns="8.8.8.8;"
fi
# use that to write new /etc/named.conf
# configure named for local queries, set forwarders and disable dnssec
(
cat <<ENDOFNAMEDCONF
//
// named.conf
//
// Provided by Red Hat bind package to configure the ISC BIND named(8) DNS
// server as a caching only nameserver (as a localhost DNS resolver only).
//
// See /usr/share/doc/bind*/sample/ for example named configuration files.
//

options {
        listen-on port 53 { 127.0.0.1; };
        listen-on-v6 port 53 { ::1; };
        forwarders { $ndns };
        forward only;
        directory       "/var/named";
        dump-file       "/var/named/data/cache_dump.db";
        statistics-file "/var/named/data/named_stats.txt";
        memstatistics-file "/var/named/data/named_mem_stats.txt";
        allow-query     { localhost; };
        recursion yes;

        dnssec-enable no;
        dnssec-validation no;
        dnssec-lookaside auto;

        /* Path to ISC DLV key */
        bindkeys-file "/etc/named.iscdlv.key";
};

logging {
        channel default_debug {
                file "data/named.run";
                severity dynamic;
        };
};

zone "." IN {
        type hint;
        file "named.ca";
};

include "/etc/named.rfc1912.zones";

ENDOFNAMEDCONF
) > /etc/named.conf.new
# put new named.conf in place
rm -f /etc/named.conf.old
ln /etc/named.conf /etc/named.conf.old
mv -f /etc/named.conf.new /etc/named.conf
chown root:named /etc/named.conf
chmod 640 /etc/named.conf

# make localhost the primary nameserver
# leave other defined nameservers in case something bad happens
# e.g., named does not start on boot
if ! grep -E '^[[:space:]]*nameserver[[:space:]]*127.0.0.1' /etc/resolv.conf > /dev/null
then
    echo 'nameserver 127.0.0.1' > /etc/resolv.conf.new
    cat /etc/resolv.conf >> /etc/resolv.conf.new
    rm -f /etc/resolv.conf.old
    ln /etc/resolv.conf /etc/resolv.conf.old
    mv -f /etc/resolv.conf.new /etc/resolv.conf
fi

# if a host uses dhclient to acquire an IP address, we need to make sure 127.0.0.1 sticks around after a reboot, or renewal
# also, trying to keep 127.0.0.1 at the top of the nameserver list
if [ -d /etc/dhcp ]
then
    if ! grep -E 'prepend[[:space:]]*domain-name-servers[[:space:]]*.*127\.0\.0\.1' /etc/dhcp/dhclient.conf > /dev/null 2> /dev/null
    then
        echo 'prepend domain-name-servers 127.0.0.1 ;' >> /etc/dhcp/dhclient.conf
    fi
fi

# NetworkManager is a bit more prickly
# Here, we assume that eth0 is the interface we want. YMMV.
if [ -d /etc/dhcp ]
then
    if ! grep -E 'prepend[[:space:]]*domain-name-servers[[:space:]]*.*127\.0\.0\.1' /etc/dhcp/dhclient-eth0.conf > /dev/null 2> /dev/null
    then
        echo 'prepend domain-name-servers 127.0.0.1 ;' >> /etc/dhcp/dhclient-eth0.conf
    fi
fi


# restart named and make sure it will start on boot
/sbin/service named restart
/sbin/chkconfig named on


%postun
if [ "$1" = "0" ] ; then
    # undo changes to /etc/dhcp/dhclient-eth0.conf
    grep -vE 'prepend[[:space:]]*domain-name-servers[[:space:]]*.*127\.0\.0\.1' /etc/dhcp/dhclient-eth0.conf > /etc/dhcp/dhclient-eth0.conf.new
    ln /etc/dhcp/dhclient-eth0.conf /etc/dhcp/dhclient-eth0.conf.saved
    mv -f /etc/dhcp/dhclient-eth0.conf.new /etc/dhcp/dhclient-eth0.conf
    # undo changes to /etc/dhcp/dhclient.conf
    grep -vE 'prepend[[:space:]]*domain-name-servers[[:space:]]*.*127\.0\.0\.1' /etc/dhcp/dhclient.conf > /etc/dhcp/dhclient.conf.new
    ln /etc/dhcp/dhclient.conf /etc/dhcp/dhclient.conf.saved
    mv -f /etc/dhcp/dhclient.conf.new /etc/dhcp/dhclient.conf

    # undo changes to /etc/resolv.conf
	rm -f /etc/resolv.conf.old
	rm -f /etc/resolv.conf.new
	grep -vE "^[[:space:]]*nameserver 127.0.0.1" /etc/resolv.conf > /etc/resolv.conf.new
	ln /etc/resolv.conf /etc/resolv.conf.old
	mv -f /etc/resolv.conf.new /etc/resolv.conf
fi


%changelog
* Mon May 23 2011 Jason Harrington <jason@fnal.gov> 2.0-2
- Fixed bugs dealing with NetworkManager comments in /etc/resolv.conf
- Added default DNS forwarder when no nameserver list found
- Added eth0-specific domain-name-server prepend for use with NetworkManager
- Added post-uninstall bits to undo dhclient tweaks

* Thu May 19 2011 Jason Harrington <jason@fnal.gov> 2.0-1
- Updated for use with SL(F)6x

*Fri Jan 05 2007 Troy Dawson <dawson@fnal.gov> 1.0-2
- Added post and postun to add 127.0.0.1 to the resolv.conf

*Fri Jan 05 2007 Troy Dawson <dawson@fnal.gov> 1.0-1
- First pass, looks good
